import { cassandra as config } from "@config";
import { logger } from "@creations.works/logger";
import { Client, auth } from "cassandra-driver";

class CassandraService {
	private static instance: Client | null = null;

	private constructor() {}

	public static getClient(): Client {
		if (!CassandraService.instance) {
			logger.error("Cassandra client is not initialized somehow?");
			process.exit(1);
		}
		return CassandraService.instance;
	}

	public static async connect({ withKeyspace = true } = {}): Promise<void> {
		if (CassandraService.instance) return;

		const client = new Client({
			contactPoints: config.contactPoints,
			localDataCenter: config.datacenter,
			authProvider: config.authEnabled
				? new auth.PlainTextAuthProvider(config.username, config.password)
				: undefined,
			protocolOptions: {
				port: config.port,
			},
			...(withKeyspace && config.keyspace ? { keyspace: config.keyspace } : {}),
		});

		try {
			await client.connect();
			logger.info("Connected to Cassandra successfully.");
			CassandraService.instance = client;
		} catch (error) {
			logger.error(["Failed to connect to Cassandra:", error as Error]);
			process.exit(1);
		}
	}

	public static async shutdown(): Promise<void> {
		if (CassandraService.instance) {
			await CassandraService.instance.shutdown();
			logger.info("Cassandra client shut down.");
			CassandraService.instance = null;
		}
	}
}

export { CassandraService as cassandra };
